# vim: set fileencoding=utf-8 :
from __future__ import absolute_import, division

# vim: set fileencoding=utf-8 :
from __future__ import absolute_import, division
from dictalchemy.tests import (
    TestCase,
    Named,
    NamedWithOtherPk,
    OneToManyChild,
    OneToManyParent,
    WithHybrid,
    WithDefaultInclude
)


class TestFromdict(TestCase):

    def test_simple(self):
        s = self.session
        named = Named('a name')
        s.add(named)
        s.commit()
        new = {'name': 'other name'}
        named.fromdict(new)
        assert named.asdict() == {'id': named.id, 'name': new['name']}

    def test_simple_aborts_on_pk(self):
        s = self.session
        named = Named('a name')
        s.add(named)
        s.commit()
        new = {'id': 7}
        try:
            named.fromdict(new)
        except:
            pass
        else:
            assert False

    def test_fromdict_allow_pk(self):
        s = self.session
        named = Named('a name')
        s.add(named)
        s.commit()
        new = {'id': 7}
        named.fromdict(new, allow_pk=True)
        assert named.asdict() == {'id': new['id'], 'name': named.name}

    def test_simple_aborts_on_named_pk_1(self):
        s = self.session
        named = NamedWithOtherPk('a name')
        s.add(named)
        s.commit()
        new = {'id': 7}
        try:
            named.fromdict(new)
            assert False
        except Exception, e:
            assert True, str(e)

    def test_one_to_many_plain(self):
        child = OneToManyChild('child')
        self.session.add(child)
        parent = OneToManyParent('parent')
        parent.child = child
        self.session.add(parent)
        self.session.commit()

        parent.fromdict({'name': 'Parent new name',
                         'child': {'name': 'Child new name'}},
                        follow=['child'])
        result = parent.asdict(follow=['child'])
        expected = {'id': parent.id,
                    'name': 'Parent new name',
                    'child': {'id': child.id, 'name': 'Child new name'}}

        assert result == expected, "%r == %r" % (result, expected)

    def test_exclude(self):
        h = Named('a name')
        h.fromdict({'name': 'other name'}, exclude=['name'])
        assert h.name == 'a name'

    def test_include(self):
        h = WithHybrid(1)
        h.fromdict({'id': 17}, include=['id'])
        assert h.id == 17

    def test_dictalchemy_include(self):
        m = WithHybrid(2)
        m.fromdict({'id': 7})
        assert m.id == 2
        setattr(m, 'dictalchemy_include', ['id'])
        m.fromdict({'id': 7})
        assert m.id == 7

    def test_dictalchemy_asdict_include_overrides(self):
        m = WithHybrid(2)
        m.fromdict({'id': 7})
        assert m.id == 2
        setattr(m, 'dictalchemy_include', ['id'])
        m.fromdict({'id': 7})
        setattr(m, 'dictalchemy_fromdict_include', [])
        m.fromdict({'id': 2})
        assert m.id == 7

    def test_default_include(self):
        h = WithDefaultInclude(1)
        h.fromdict({'id_alias': 4})
        assert h.id_alias == 4

    def test_only(self):
        named = Named('a name')
        named.fromdict({'name': 'other name'}, only=['name'])
        assert named.name == 'other name'
        named.fromdict({'name': 'other name'}, only=['id'])
        assert named.name == 'other name'

    def test_only_overrides_include(self):
        named = Named('a name')
        named.fromdict({'name': 'other name'}, include='name', only=['id'])
        assert named.name == 'a name'

    def test_only_overrides_exclude(self):
        named = Named('a name')
        named.fromdict({'name': 'other name'}, exclude='name', only=['name'])
        assert named.name == 'other name'



# vim: set fileencoding=utf-8 :
from __future__ import absolute_import, division

import unittest
import dictalchemy.tests as tests
from dictalchemy import make_class_dictable
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy import Column, String, Integer

engine = create_engine('sqlite:///:memory:', echo=False)
from sqlalchemy.ext.declarative import declarative_base
Base = declarative_base(engine)


class MakeClassDictable(Base):

    __tablename__ = 'makeclassdictable'

    id = Column(Integer, primary_key=True)

    name = Column(String)

    def __init__(self, name):
        self.name = name


class TestAsdict(unittest.TestCase):

    def setUp(self):
        """ Recreate the database """
        Base.metadata.create_all(engine)
        Session = sessionmaker(bind=engine)
        self.session = Session()

    def tearDown(self):
        Base.metadata.drop_all()

    def test_make_class_dictable(self):
        assert not hasattr(MakeClassDictable, 'asdict')
        m = MakeClassDictable('dictable')
        self.session.add(m)
        self.session.commit()

        assert not hasattr(m, 'asdict')
        make_class_dictable(MakeClassDictable)
        assert m.asdict() == {'id': m.id, 'name': m.name}


class TestMakeDictable(tests.TestCase):

    def test_dict(self):
        named = tests.Named('a name')
        self.session.add(named)
        self.session.commit()
        assert dict(named) == {'id': named.id, 'name': 'a name'}



# vim: set fileencoding=utf-8 :
from __future__ import absolute_import, division

from dictalchemy import DictableModel
import unittest

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy import Table, Column, String, Integer, ForeignKey
from sqlalchemy.orm import relationship, backref, synonym
from sqlalchemy.ext.hybrid import hybrid_property
from sqlalchemy.orm.collections import attribute_mapped_collection


# Setup sqlalchemy
engine = create_engine('sqlite:///:memory:', echo=False)
from sqlalchemy.ext.declarative import declarative_base
Base = declarative_base(engine, cls=DictableModel)


class TestCase(unittest.TestCase):

    def setUp(self):
        """ Recreate the database """
        Base.metadata.create_all(engine)
        Session = sessionmaker(bind=engine)
        self.session = Session()

    def tearDown(self):
        Base.metadata.drop_all()


class Named(Base):
    __tablename__ = 'named'
    id = Column(Integer, primary_key=True)
    name = Column(String)

    def __init__(self, name):
        self.name = name


class NamedWithOtherPk(Base):
    __tablename__ = 'namedwithotherpk'
    id = Column('other_id', Integer, primary_key=True)
    name = Column(String)

    def __init__(self, name):
        self.name = name


class NamedOtherColumnName(Base):
    __tablename__ = 'named_with_other_column'

    id = Column(Integer, primary_key=True)
    name = Column('namecolumn', String)

    def __init__(self, name):
        self.name = name


class NamedWithSynonym(Base):
    __tablename__ = 'named_with_synonym'

    id = Column(Integer, primary_key=True)
    _name = Column(String)

    def _setname(self, name):
        self._name = name

    def _getname(self):
        return self._name

    name = synonym('_name', descriptor=property(_getname, _setname))

    def __init__(self, name):
        self.name = name


class OneToManyChild(Base):

    __tablename__ = 'onetomanychild'

    id = Column(Integer, primary_key=True)

    name = Column(String)

    def __init__(self, name):
        self.name = name


class OneToManyParent(Base):

    __tablename__ = 'onetomanyparent'

    id = Column(Integer, primary_key=True)

    name = Column(String)

    _child_id = Column(Integer, ForeignKey(OneToManyChild.id))

    child = relationship(OneToManyChild,
                         primaryjoin=_child_id == OneToManyChild.id,
                         backref=backref('parent'))

    def __init__(self, name):
        self.name = name


m2m_table = Table('m2m_table',
                  Base.metadata,
                  Column('left_id', Integer,
                         ForeignKey('m2mleft.id'),
                         primary_key=True),
                  Column('right_id', Integer,
                         ForeignKey('m2mright.id'),
                         primary_key=True),
                  )


class M2mLeft(Base):
    __tablename__ = 'm2mleft'

    id = Column(Integer, primary_key=True)

    name = Column(String)

    def __init__(self, name):
        self.name = name

    rights = relationship('M2mRight',
                          secondary=m2m_table,
                          backref=backref('lefts'))


class M2mRight(Base):
    __tablename__ = 'm2mright'

    id = Column(Integer, primary_key=True)

    name = Column(String)

    def __init__(self, name):
        self.name = name


class MultipleChildChild1Child(Base):

    __tablename__ = 'multiplechildchild1child'

    id = Column(Integer, primary_key=True)

    name = Column(String)

    def __init__(self, name):
        self.name = name


class MultipleChildChild1(Base):

    __tablename__ = 'multiplechildchild1'

    id = Column(Integer, primary_key=True)

    name = Column(String)

    _child_id = Column(Integer, ForeignKey(MultipleChildChild1Child.id))

    child = relationship(MultipleChildChild1Child,
                         primaryjoin=_child_id == MultipleChildChild1Child.id,
                         backref=backref('parent'))

    def __init__(self, name):
        self.name = name


class MultipleChildChild2(Base):

    __tablename__ = 'multiplechildchild2'

    id = Column(Integer, primary_key=True)

    name = Column(String)

    def __init__(self, name):
        self.name = name


class MultipleChildParent(Base):

    __tablename__ = 'multiplechildparent'

    id = Column(Integer, primary_key=True)

    name = Column(String)

    _child1_id = Column(Integer, ForeignKey(MultipleChildChild1.id))

    _child2_id = Column(Integer, ForeignKey(MultipleChildChild2.id))

    child1 = relationship(MultipleChildChild1,
                          primaryjoin=_child1_id == MultipleChildChild1.id,
                          backref=backref('parent'))

    child2 = relationship(MultipleChildChild2,
                          primaryjoin=_child2_id == MultipleChildChild2.id,
                          backref=backref('parent'))

    def __init__(self, name):
        self.name = name


class WithHybrid(Base):

    __tablename__ = 'withhybrid'

    _id = Column('id', Integer, primary_key=True)

    @hybrid_property
    def id(self):
        return self._id

    @id.setter
    def set_id(self, value):
        self._id = value

    def __init__(self, id):
        self.id = id


class WithDefaultInclude(Base):

    __tablename__ = 'withdefaultinclude'

    dictalchemy_include = ['id_alias']

    id = Column('id', Integer, primary_key=True)

    @hybrid_property
    def id_alias(self):
        return self.id

    @id_alias.setter
    def set_id_alias(self, value):
        self.id = value

    def __init__(self, id):
        self.id = id


class WithAttributeMappedCollectionChild(Base):

    __tablename__ = 'withattributemappedcollectionchild'

    id = Column(Integer, primary_key=True)

    name = Column(String, unique=True, nullable=False)

    parent_id = Column(Integer, ForeignKey('withattributemappedcollection.id'))

    def __init__(self, name):
        self.name = name


class WithAttributeMappedCollection(Base):

    __tablename__ = 'withattributemappedcollection'

    id = Column(Integer, primary_key=True)

    childs = relationship(WithAttributeMappedCollectionChild,
                          collection_class=attribute_mapped_collection('name'),
                          cascade="all, delete-orphan",
                          backref=backref('parents'))



# vim: set fileencoding=utf-8 :
"""

************
Introduction
************

This library adds basic functionality for getting a dict from an SQLAlchemy
model and updating a model from a dict.

SQLAlchemy is a very complex library. It contains synonyms, lists, sets, mixins
and god knows what. Automatically convert that to a dict or update from a
dict is not a simple task. This library should not be used on more complex
models without thorough testing, it should however be fine to use on simple
models.

Usage
=====

There are two ways to use dictalchemy. Either by using
:class:`dictalchemy.classes.DictableModel` as base class or by using
:meth:`dictalchemy.utils.make_class_dictable` on an existing base class.

The actual work is done in the functions :func:`dictalchemy.utils.asdict` and
:func:`dictalchemy.utils.fromdict`.

Since attributes are checked on instances each instance can get their own setup
of rules. This can be useful when returning a model instance as a response. If
default values are set on that specific instance calling `dict` will render it
properly.

Using DictableModel
-----------------------------------------

Use :class:`dictalchemy.classes.DictableModel` as a base class for
:class:`sqlalchemy.ext.declarative_base`.

Example::

    from sqlalchemy.ext import declarative_base
    from dictalchemy import DictableModel
    Base = declarative_base(cls=DictableModel)

Using make_class_dictable
-------------------------

:func:`dictalchemy.utils.make_class_dictable` adds methods and attributes to an
already existing class

Example::

    from sqlalchemy.ext import declarative_base
    from dictalchemy import make_class_dictable
    base = declarative_base()
    make_class_dictable(base)

Attributes and parameters
-------------------------

Dictalchemy uses some basic attributes and parameters to convert to and from
dict. The most basic are:

* include
* exclude
* exclude_pk
* allow_pk
* follow
* only

The defaults varies depending on the flag. For example, allow_pk will by
default be set to True for fromdict and exclude_pk will be set to False in
asdict.

A class or model can have these attributes set, prefixed with
*dictalchemy\_*. Some of them can also be overridden depending on if
:meth:`dictalchemy.utils.asdict` or :meth:`dictalchemy.utils.fromdict` is
called. If they are set they will override the more basic attribute.

Some semi-rules
^^^^^^^^^^^^^^^

* In general `include` flags will override `exclude` flags.
* In general `only` will override `exclude` and `include` flags.
* In :meth:`dictalchemy.utils.fromdict` `allow_pk`=False will override all \
        other flags.


Class/Model attributes
^^^^^^^^^^^^^^^^^^^^^^

* dictalchemy_exclude
* dictalchemy_include
* dictalchemy_fromdict_include
* dictalchemy_asdict_include
* dictalchemy_exclude_underscore
* dictalchemy_fromdict_allow_pk


A note about synonyms
=====================

Synonyms wraps a reader and a writer method. These methods can do whatever they
want to so there is no way to safely updated data with synonyms. So keep in
mind that using synonyms will make it possible to circumvent for example
dictalchemy_exclude_pk.

Bugs and missing features
=========================

This library is developed as a helper-library to another piece of software.
What works for that software is also implemented and tested in dictalchemy.

There are bugs and missing features in this library. Please don't hesitate to
register issues at `github <https://github.com/danielholmstrom/dictalchemy>`_.

"""
from __future__ import absolute_import, division

from dictalchemy.classes import DictableModel
from dictalchemy.utils import make_class_dictable, asdict
from dictalchemy.errors import (DictalchemyError, UnsupportedRelationError,
                                MissingRelationError)

__all__ = [DictableModel,
           make_class_dictable,
           asdict,
           DictalchemyError,
           UnsupportedRelationError,
           MissingRelationError]



# vim: set fileencoding=utf-8 :
"""
~~~~~~~~~
Utilities
~~~~~~~~~
"""
from __future__ import absolute_import, division

from sqlalchemy.orm import (RelationshipProperty, ColumnProperty,
                            SynonymProperty)
from sqlalchemy.orm.collections import InstrumentedList, MappedCollection

from dictalchemy import constants
from dictalchemy import errors


def get_relation_keys(model):
    """Get relation keys for a model

    :returns: List of RelationProperties
    """
    return [k.key for k in model.__mapper__.iterate_properties if
            isinstance(k, RelationshipProperty)]


def get_column_keys(model):
    """Get column keys for a model

    :returns: List of column keys
    """
    return [k.key for k in model.__mapper__.iterate_properties if
            isinstance(k, ColumnProperty)]


def get_synonym_keys(model):
    """Get synonym keys for a model

    :returns: List of keys for synonyms
    """
    return [k.key for k in model.__mapper__.iterate_properties if
            isinstance(k, SynonymProperty)]


def get_primary_key_properties(model):
    """Get the column properties that affects a primary key

    :returns: Set of column keys
    """
    # Find primary keys
    primary_keys = set()
    for k in model.__mapper__.iterate_properties:
        if hasattr(k, 'columns'):
            for c in k.columns:
                if c.primary_key:
                    primary_keys.add(k.key)
    return primary_keys


def asdict(model, exclude=None, exclude_underscore=None, exclude_pk=None,
           follow=None, include=None, only=None):
    """Get a dict from a model

    Simple example::

        session.query(User).asdict()
        {'id': 1, 'username': 'Gerald'}

    Using exclude_pk::

        session.query(User).asdict(exclude_pk=True)
        {'username': 'Gerald'}

    Using exclude::

        session.query(User).asdict(exclude=['id'])
        {'username': 'Gerald'}

    Using follow without arguments::

        session.query(User).asdict(follow={'groups':{}})
        {'username': 'Gerald', groups=[{'id': 1, 'name': 'User'}]}

    Using follow with arguments::

        session.query(User).asdict(follow={'groups':{'exclude': ['id']})
        {'username': 'Gerald', groups=[{'name': 'User'}]}

    Using include(for example for including synonyms/properties)::

        session.query(User).asdict(include=['displayname']
        {'id': 1, 'username': 'Gerald', 'displayname': 'Gerald'}

    :param follow: List or dict of relationships that should be followed. \
            If the parameter is a dict the value should be a dict of \
            keyword arguments. Currently it follows InstrumentedList,\
            MappedCollection and regular 1:1, 1:m, m:m relationships.
    :param exclude: List of properties that should be excluded, will be \
            merged with `model.classes.DictableModel.dictalchemy_exclude`
    :param exclude_pk: If True any column that refers to the primary key will \
            be excluded.
    :param exclude_underscore: Overides `model.dictalchemy_exclude_underscore`\
            if set
    :param include: List of properties that should be included. Use this to \
            allow python properties to be called. This list will be merged \
            with `model.dictalchemy_asdict_include` or \
            `model.dictalchemy_include`.
    :param only: List of properties that should be included. This will \
            override everything else except `follow`.

    :raises: :class:`dictalchemy.errors.MissingRelationError` \
            if `follow` contains a non-existent relationship.
    :raises: :class:`dictalchemy.errors.UnsupportedRelationError` If `follow` \
            contains an existing relationship that currently isn't supported.

    :returns: dict

    """

    if follow is None:
        follow = []
    try:
        follow = dict(follow)
    except ValueError:
        follow = dict.fromkeys(list(follow), {})

    columns = get_column_keys(model)
    synonyms = get_synonym_keys(model)
    relations = get_relation_keys(model)

    if only:
        attrs = only
    else:
        exclude = exclude or []
        exclude += getattr(model, 'dictalchemy_exclude',
                           constants.default_exclude) or []
        if exclude_underscore is None:
            exclude_underscore = getattr(model,
                                         'dictalchemy_exclude_underscore',
                                         constants.default_exclude_underscore)
        if exclude_underscore:
            # Exclude all properties starting with underscore
            exclude += [k.key for k in model.__mapper__.iterate_properties
                        if k.key[0] == '_']
        if exclude_pk is True:
            exclude += get_primary_key_properties(model)

        include = (include or []) + (getattr(model,
                                             'dictalchemy_asdict_include',
                                             getattr(model,
                                                     'dictalchemy_include',
                                                     None)) or [])
        attrs = [k for k in columns + synonyms + include if k not in exclude]

    data = dict([(k, getattr(model, k)) for k in attrs])

    for (k, args) in follow.iteritems():
        if k not in relations:
            raise errors.MissingRelationError(k)
        rel = getattr(model, k)
        if hasattr(rel, 'asdict'):
            data.update({k: rel.asdict(**args)})
        elif isinstance(rel, InstrumentedList):
            children = []
            for child in rel:
                if hasattr(child, 'asdict'):
                    children.append(child.asdict(**args))
                else:
                    children.append(dict(child))
            data.update({k: children})
        elif isinstance(rel, MappedCollection):
            children = {}
            for (child_key, child) in rel.iteritems():
                if hasattr(child, 'asdict'):
                    children[child_key] = child.asdict(**args)
                else:
                    children[child_key] = child.dict(child)
            data.update({k: children})
        else:
            raise errors.UnsupportedRelationError(k)

    return data


def fromdict(model, data, exclude=None, exclude_underscore=None,
             allow_pk=None, follow=None, include=None, only=None):
    """Update a model from a dict

    Works almost identically as :meth:`dictalchemy.utils.asdict`. However, it
    will not create missing instances or update collections.

    This method updates the following properties on a model:

    * Simple columns
    * Synonyms
    * Simple 1-m relationships

    :param data: dict of data
    :param exclude: list of properties that should be excluded
    :param exclude_underscore: If True underscore properties will be excluded,\
            if set to None model.dictalchemy_exclude_underscore will be used.
    :param allow_pk: If True any column that refers to the primary key will \
            be excluded. Defaults model.dictalchemy_fromdict_allow_pk or \
            dictable.constants.fromdict_allow_pk. If set to True a primary \
            key can still be excluded with the `exclude` parameter.
    :param follow: Dict of relations that should be followed, the key is the \
            arguments passed to the relation. Relations only works on simple \
            relations, not on lists.
    :param include: List of properties that should be included. This list \
            will override anything in the exclude list. It will not override \
            allow_pk.
    :param only: List of the only properties that should be returned. This \
            will not override `allow_pk` or `follow`.

    :raises: :class:`dictalchemy.DictalchemyError` If a primary key is \
            in data and allow_pk is False

    :returns: The model

    """

    if follow is None:
        follow = []
    try:
        follow = dict(follow)
    except ValueError:
        follow = dict.fromkeys(list(follow), {})

    columns = get_column_keys(model)
    synonyms = get_synonym_keys(model)
    relations = get_relation_keys(model)
    primary_keys = get_primary_key_properties(model)

    if only:
        attrs = only
    else:
        exclude = exclude or []
        exclude += getattr(model, 'dictalchemy_exclude',
                           constants.default_exclude) or []
        if exclude_underscore is None:
            exclude_underscore = getattr(model,
                                         'dictalchemy_exclude_underscore',
                                         constants.default_exclude_underscore)

        if exclude_underscore:
            # Exclude all properties starting with underscore
            exclude += [k.key for k in model.__mapper__.iterate_properties
                        if k.key[0] == '_']

        if allow_pk is None:
            allow_pk = getattr(model, 'dictalchemy_fromdict_allow_pk',
                               constants.default_fromdict_allow_pk)

        include = (include or []) + (getattr(model,
                                             'dictalchemy_fromdict_include',
                                             getattr(model,
                                                     'dictalchemy_include',
                                                     None)) or [])
        attrs = [k for k in columns + synonyms if k not in exclude] + include

    # Update simple data
    for k, v in data.iteritems():
        if not allow_pk and k in primary_keys:
            msg = "Primary key(%r) cannot be updated by fromdict."
            "Set 'dictalchemy_fromdict_allow_pk' to True in your Model"
            " or pass 'allow_pk=True'." % k
            raise errors.DictalchemyError(msg)
        if k in attrs:
            setattr(model, k, v)

    # Update simple relations
    for (k, args) in follow.iteritems():
        if k not in data:
            continue
        if k not in relations:
            raise errors.MissingRelationError(k)
        rel = getattr(model, k)
        # TODO: Check for fromdict, not asdict
        if hasattr(rel, 'asdict'):
            rel.fromdict(data[k], **args)

    return model


def iter(model):
    """iter method for models"""
    for i in model.asdict().iteritems():
        yield i


def make_class_dictable(
        cls,
        exclude=constants.default_exclude,
        exclude_underscore=constants.default_exclude_underscore,
        fromdict_allow_pk=constants.default_fromdict_allow_pk,
        include=None,
        asdict_include=None,
        fromdict_include=None):
    """Make a class dictable

    Useful for when the Base class is already defined, for example when using
    Flask-SQLAlchemy.

    Warning: This method will overwrite existing attributes if they exists.

    :param exclude: Will be set as dictalchemy_exclude on the class
    :param exclude_underscore: Will be set as dictalchemy_exclude_underscore \
            on the class
    :param fromdict_allow_pk: Will be set as dictalchemy_fromdict_allow_pk\
            on the class
    :param include: Will be set as dictalchemy_include on the class.
    :param asdict_include: Will be set as `dictalchemy_asdict_include` on the \
            class. If not None it will override `dictalchemy_include`.
    :param fromdict_include: Will be set as `dictalchemy_fromdict_include` on \
            the class. If not None it will override `dictalchemy_include`.

    :returns: The class
    """

    setattr(cls, 'dictalchemy_exclude', exclude)
    setattr(cls, 'dictalchemy_exclude_underscore', exclude_underscore)
    setattr(cls, 'dictalchemy_fromdict_allow_pk', fromdict_allow_pk)
    setattr(cls, 'asdict', asdict)
    setattr(cls, 'fromdict', fromdict)
    setattr(cls, '__iter__', iter)
    setattr(cls, 'dictalchemy_include', include)
    setattr(cls, 'dictalchemy_asdict_include', asdict_include)
    setattr(cls, 'dictalchemy_fromdict_include', fromdict_include)
    return cls



"""
~~~~~~~~~~~
Dictalchemy
~~~~~~~~~~~

Contains asdict() and fromdict() methods that will work on SQLAlchemy
declarative models.

Read more in the source or on github
<https://github.com/danielholmstrom/dictalchemy>.
"""

import os
from setuptools import find_packages
from distutils.core import setup

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()

# Requirements for the package
install_requires = [
    'SQLAlchemy>=0.8.0',
]

# Requirement for running tests
test_requires = install_requires

setup(name='dictalchemy',
      version='0.1.1',
      description="Contains asdict and fromdict methods for SQL-Alchemy "
      "declarative models",
      long_description=README,
      url='http://github.com/danielholmstrom/dictalchemy/',
      license='MIT',
      author='Daniel Holmstrom',
      author_email='holmstrom.daniel@gmail.com',
      platforms='any',
      classifiers=['Development Status :: 5 - Production/Stable',
                   'License :: OSI Approved :: MIT License',
                   'Environment :: Web Environment',
                   'Intended Audience :: Developers',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
                   'Topic :: Software Development :: '
                   'Libraries :: Python Modules'],
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      tests_require=test_requires,
      test_suite='dictalchemy',
      )



# vim: set fileencoding=utf-8 :
"""
~~~~~~~
Classes
~~~~~~~

Contains :class:`DictableModel` that can be used as a base class for
:meth:`sqlalchemy.ext.declarative_base`.

"""

from __future__ import absolute_import, division

from dictalchemy import utils


class DictableModel(object):
    """Can be used as a base class for :meth:`sqlalchemy.ext.declarative`

    Contains the methods :meth:`DictableModel.__iter__`,
    :meth:`DictableModel.asdict` and :meth:`DictableModel.fromdict`.

    Example usage:

        >>> from dictalchemy import DictableModel
        >>> from slqlachemy.ext.declarative import declarative_base
        >>> Base = declarative_base(cls=DictableModel)


    :ivar dictalchemy_exclude: List of properties that should always be \
            excluded.
    :ivar dictalchemy_exclude_underscore: If True properties starting with an \
            underscore will always be excluded.
    :ivar dictalchemy_fromdict_allow_pk: If True the primary key can be \
            updated by :meth:`DictableModel.fromdict`.
    :ivar dictalchemy_asdict_include: List of properties that should always \
            be included when calling :meth:`DictableModel.asdict`
    :ivar dictalchemy_fromdict_include: List of properties that should always \
            be included when calling :meth:`DictableModel.fromdict`

    """

    asdict = utils.asdict

    fromdict = utils.fromdict

    __iter__ = utils.iter
